import time
from epics import caput, caget, pv
import numpy as np


# Trigger Sync
class TriggerSynchronisation:
    
    triggerPV = pv.PV("AWATrigCtrl:SetCh8") # used to trigger
    triggerCheckPV = None # PV used to check if the trigger works as expected

    # the ICT current is a good default check since it updates with every bunchtrain
    def setup(self, checkPVName = "AWAVXI11ICT:Ch1"):
        self.triggerPV = pv.PV("AWATrigCtrl:SetCh8")
        self.triggerCheckPV = pv.PV(checkPVName)
        self.triggerPV.put(1)
        

    def check_trigger_sync(self):
        temp = self.triggerCheckPV.get() # do not think this is strictly necessary
        ts_ict_ts_b = self.triggerCheckPV.timestamp
        time.sleep(1.1) # need to be larger than pulse
        ts_ict_ts_a = self.triggerCheckPV.timestamp

        # if the time difference is large enough and PV timestamp did not change 
        # we can assume that the triggerControl is setup correctlys
        # and the FrameGrabber and Osciloscopes are waiting for TriggerPV.put(1)
        # Note: 0.0000001 is a magic number that might need to be changed for robust function
        if ((ts_ict_ts_a - ts_ict_ts_b) >  0.000001): 
            print("Please, Make sure the osciloscopes and the Images are frozen after each trigger.")
            input("Press Enter to continue...")
            self.check_trigger_sync()
        pass

    def trigger(self):
        temp = self.triggerCheckPV.get()
        ts_ict_ts_b =self.triggerCheckPV.timestamp # before trigger
        
        # trigger scopes and camera update
        self.triggerPV.put(1)
        
        ts_ict_ts_a = self.triggerCheckPV.timestamp #after trigger

        n_waits = 0
        
        # making sure the trigger happend
        # if the time difference is zero the trigger is still waiting 
        # for the next machine pulse or somethingelse
        # Note: 0.0000001 is a magic number that might need to be changed for robust function
        while ((ts_ict_ts_a - ts_ict_ts_b) < 0.000001 ):
            # retrigger if time sptampe differnce was to small by chance
            if (n_waits > 60): # this number depends on pulsefrequency
                print("Re-Triggering...")
                self.triggerPV.put(1)
                n_waits = 0
            time.sleep(0.02)
            n_waits+=1
            ts_ict_ts_a = self.triggerCheckPV.timestamp

#####

# parts of evaluate or get observables

# not participating in triggersynchronisation
def get_GunPVs():
    results = {}
    
    results["gun_waveform"] = np.array(caget("AWALLRF:WF:K1")) ### move some other place. xopt doesnt like lists
    # get laser phase
    results["laser_phase_rb"] = caget("TST24:Ch3:Phase")
    
    ## get the RF info
    results["gun_phase_meas"] = caget("AWALLRF:K1:MeasuredPhase")
    results["gun_amp"] = caget("AWALLRF:K1:MeasuredAmp")
    results["gun_phase_rb"] = caget("AWALLRF:K1:SetPhaseRB")
    results["K1_phase_readback"] = caget("AWALLRF:K1:Phase")
    #results["GUN_amp_TS"] = pv_gun_amp.timestamp
    results["LLRF1set"] = caget("AWA:DAC0:Ch08")
    return results


# not participating in triggersynchronisation
# only up to DS1-4
def get_DSs():
    results = {}
    # DS
    results["DS1RB"] = caget("AWA:Drive:DS1:RB")
    results["DS3RB"] = caget("AWA:Drive:DS3:RB")
    results["DS4RB"] = caget("AWA:Drive:DS4:RB")

    return results
def get_ICT1():
    results = {}
    results["ICT1"] = caget("AWAVXI11ICT:Ch1")
    results["ICT1_nC"] = caget("AWAVXI11ICT:Ch1") * 1e9
    return results

# if hocked up to the Osciloscope
def get_FCT1(): 
    results = {}
    results["FCT1"] = caget("AWAVXI11ICT:Ch3")
    results["FCT1_nC"] = caget("AWAVXI11ICT:Ch3") * 1e9
    return results

def get_waveforms(fct12ghz=False):
    ### save waveforms to metadata/h5files since Xopt is not happy with lists of different lengths
    results = {}
    results["gun_waveform"] = np.array(caget("AWALLRF:WF:K1"))
    # results["gunpulsewf"] = np.array(caget("AWABPPM:wf1")) # usually not on this osciloscope
    results["ICT1wf"] = np.array(caget("AWAVXI11ICT:rawwf1"))
    if fct12ghz:
         #if hocked up to osciloscope 
        results["FCT1wf_12.5GHz"] = np.array(caget("AWAVXI11:Tek1:wf1"))
    else:
        #if hocked up to osciloscope 
        results["FCT1wf"] = np.array(caget("AWAVXI11ICT:rawwf3"))
    
    return results
    


