import socket

import time
from pathlib import Path

import h5py
import numpy as np
from epics import caget, caput
from interface import AWAInterface


nshots = 5  # number of shots per settings
nreali = 100  # number of setup realizations (or settings)
directory = "./run100/"
rootname = "sol_dt1_gunPhi_100conf"


q_range = 0.3


ict = 0
for i in range(nshots):
    caput("AWATrigCtrl:GetCh8.PROC", 1)
    while caget("AWATrigCtrl:GetCh8") > 0:
        caput("AWATrigCtrl:GetCh8.PROC", 1)
    ict_tmp = np.abs(caget("AWAVXI11ICT:Ch1"))
    caput("AWATrigCtrl:SetCh8", 1)
    ict = ict + ict_tmp
    print(ict_tmp)
ict_mean = ict / nshots

print("average charge [nC]", ict_mean * 1e9)


calDS1 = 1
calDS3 = 1
calDT1 = 0.0

phi0 = 120.0
# param = sol1, sol2,L1dx, L1dy, phiG, AmpG, phiL1, AmpL1,
params = [
    "AWA:Drive:DS1:Ctrl",
    "AWA:Drive:DS3:Ctrl",
    "AWA:Drive:DT1V_A_N:Ctrl",
    "AWA:Drive:DT1H_A_N:Ctrl",
    "AWALLRF:K1:SetPhase",
    "AWA:K2PhAdj",
    "AWA:DAC0:Ch08",
    "AWA:DAC0:Ch09",
    "AWARPiPWM:SetPulseWidth",
]

paramsRB = [
    "AWA:Drive:DS1:Ctrl",
    "AWA:Drive:DS3:Ctrl",
    "AWA:Drive:DT1V_A_N:Ctrl",
    "AWA:Drive:DT1H_A_N:Ctrl",
    "AWALLRF:K1:SetPhaseRB",
    "AWA:K2RF:PhRB",
    "AWA:DAC0:Ch08",
    "AWA:DAC0:Ch09",
    "AWARPiPWM:SetPulseWidth",
]


print(len(params))
mini = [
    390 * calDS1,
    100 * calDS3,
    -100 * calDT1,
    -100 * calDT1,
    phi0 - 20,
    -2,
    5.7,
    3.2,
    650,
]
maxi = [
    520 * calDS1,
    270 * calDS3,
    100 * calDT1,
    100 * calDT1,
    phi0 + 20,
    2,
    5.9,
    3.2,
    650,
]
mask = [1, 1, 0, 0, 1, 0, 1, 0, 0]
value = np.zeros((len(params), nreali))
value0 = np.zeros(len(params))

# prepare the table
for i in range(len((params))):
    value[i, :] = np.random.normal(
        (maxi[i] + mini[i]) / 2, (maxi[i] - mini[i]) / 4, nreali
    )
    value0[i] = caget(params[i])
print(value, value0)

# image acquisition
screen = AWAInterface()  # This is the default option to capture the PGcamera image

# logfile
logfile = directory + "/" + rootname + ".Log"
logfileRB = directory + "/" + rootname + ".LogRB"

fl = open(logfile, "w")
flRB = open(logfileRB, "w")

for i in params:
    fl.write(i + "\t")
fl.write("\n")

for i in range(nreali):
    print("---------- nrealization #", i)
    for j in range(len(params)):
        print(params[j], value[j, i])
        if mask[j] == 1:
            print("touching", params[j])
            caput(params[j], value[j, i])
        if mask[j] == 0:
            print("NOT touching", params[j])
        time.sleep(2)
        fl.write(str(value[j, i]) + "\t")
        flRB.write(str(caget(params[j])) + "\t")

    time.sleep(1.0)
    # take shots
    filename = directory + "/" + rootname + "_" + str(i)
    fl.write(filename + "\t")
    print("data will be save in ", filename)
    screen.save_data(
        filename,
        target_charge=ict_mean,
        charge_deviation=0.2,  # within +/- 20% deviation
        n_samples=nshots,
    )  # Number of samples

    fl.write("\n")
    flRB.write("\n")

fl.close()
flRB.close()

for i in range(len((params))):
    caput(params[i], value0[i])
# print(value0)
